from fastapi.testclient import TestClient
from larry.web import app
import json
import logging
from pydantic import BaseModel
from unittest import mock

client = TestClient(app)

class Message(BaseModel):
    question: str


def test_read_main():
    response = client.get("/")
    assert response.status_code == 200

@mock.patch("openai.ChatCompletion.create", mock.MagicMock())
@mock.patch.dict("os.environ", {"OPENAI_API_KEY": "FAKE_KEY"})
def test_generate():

    response = client.post(
        url = "/generate",
        json = {"question": "Test question"}
    )
    assert response.status_code == 200

def test_generate_error():
    response = client.post(
        url = "/generate",
        data = json.dumps({"invalid": "payload"})
    )
    logging.error(response.text)
    assert response.status_code != 200


from fastapi import FastAPI, HTTPException
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
from fastapi.requests import Request
from fastapi.responses import JSONResponse
from fastapi.encoders import jsonable_encoder
import logging
from os.path import abspath, join, dirname
import openai
import os

openai.api_key = os.getenv("OPENAI_API_KEY")

app = FastAPI()

TEMPLATE_ROOT = abspath(join(dirname(__file__), 'www'))
STATIC_ROOT = abspath(join(dirname(__file__), 'www/static'))

templates = Jinja2Templates(directory=TEMPLATE_ROOT)
app.mount('/static', StaticFiles(directory=STATIC_ROOT), 'static')


@app.get("/{rest_of_path:path}")
async def react_app(req: Request, rest_of_path: str):
    logging.info(f'Rest of path: {rest_of_path}')
    return templates.TemplateResponse(
        'index.html',
        {
            'request': req
        }
    )


@app.post("/generate")
async def generate(
    req: Request,
    message: dict
):
    try:
        logging.info(f"Question: {message['question']}")
        question = message["question"]
        response = openai.ChatCompletion.create(
            model = "gpt-3.5-turbo",
            messages=[
                {"role": "system", "content": "You are a helpful assistant, skilled in explaining any question in a detailed, factul manner, while avoiding polarizing topics."},
                {"role": "user", "content": question}
            ]
        )
        logging.info(f"Answer: {response.choices[0].message}")
        payload = jsonable_encoder(
            {
                "answer": response.choices[0].message["content"],
                "chat_history_ids": ""
            })
        
        response = JSONResponse(content=payload)
        return response
    except Exception as e:
        logging.error(str(e))
        raise HTTPException(status_code=400, detail=f"Error: {str(e)}")




from setuptools import find_packages, setup
from pathlib import Path
import larry

PACKAGE_REQUIREMENTS = [
    "fastapi>=0.95.0,<1.0.0",
    "uvicorn[standard]==0.21.1",
    "jinja2",
    "click",
    "openai==0.28.1"
]

current_dir = Path(__file__).parent.parent
long_description = (current_dir / "README.md").read_text()

setup(
    name="larry-ai",
    packages=find_packages(),
    long_description = long_description,
    include_package_data=True,
    setup_requires=["setuptools", "wheel"],
    install_requires=PACKAGE_REQUIREMENTS,
    entry_points={"console_scripts": ["larry = larry.entrypoint:main"]},
    version="0.0.1",
    authors=["Rafael Pierre"],
)


import subprocess
import logging
import click
from typing import List

def run(command: List[str]):
    process = subprocess.Popen(
        command, 
        stdout=subprocess.PIPE,
        universal_newlines=True
    )

    while True:
        output = process.stdout.readline()
        print(output.strip())
        return_code = process.poll()
        if return_code is not None:
            print('RETURN CODE', return_code)
            for output in process.stdout.readlines():
                print(output.strip())
            break

@click.command()
@click.option('--logs', default='info', help='Verbosity level, defaults to INFO')
@click.option('--host', default='0.0.0.0', help='Host Address, defaults to 0.0.0.0')
@click.option('--port', default='8000', help='HTTP Port for Backend, defaults to 8000')
def main(logs: str, host: str, port: str):
    """Runs Larry Web App."""

    logging.basicConfig(level = logs.upper())
    logging.info("Starting Larry server...")

    run(["uvicorn", "larry.web:app", "--host", host, "--port", port])


if __name__ == "__main__":
    main()


from unittest import mock
from larry.entrypoint import run


@mock.patch("larry.entrypoint.main", mock.MagicMock())
def test_entrypoint():

    from larry.entrypoint import main, run
    main()
    main.assert_called_once()

def test_run():

    run(command = ["bash", "-c", "echo", "12345678910", "&&", "sleep 3", "&&", "echo 12345678910"])
    pass
